# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'

Puppet::Type.newtype(:wls_java_home) do
  include EasyType

  desc <<-DESC
  Ensure correct java home is present in WebLogic files in specified weblogic home.
  DESC

  def self.domain_file
    '/etc/wls_domains.dat'
  end

  to_get_raw_resources do
    return [] unless File.exist?(domain_file)

    file = File.read(domain_file)
    file.split.map do |domain|
      home = domain.split(':')[1]
      java_home = java_home_for(home)
      { :name => home, :java_home => java_home }
    end
  end

  def self.java_home_for(home)
    content = File.read("#{home}/bin/setDomainEnv.sh")
    content.scan(/JAVA_HOME="(.+)"/)[0][0]
  end

  # rubocop: disable Style/OptionalBooleanParameter
  def replace_java_home(file, warning = false)
    if File.exist?(file)
      Puppet.debug "Changing java_home in #{file} from: '#{provider.java_home}' to: '#{java_home}'"
      content = File.read(file)
      content.gsub!(provider.java_home, java_home)
      File.write(file, content)
    else
      message = "Skipping change of java_home in #{file}, because file not found."
      if warning
        Puppet.warning message
      else
        Puppet.debug message
      end
    end
  end
  # rubocop: enable Style/OptionalBooleanParameter

  on_modify do
    # Nodemanager
    replace_java_home("#{name}/nodemanager/nodemanager.properties")
    # bin
    replace_java_home("#{name}/bin/setNMJavaHome.sh")
    replace_java_home("#{name}/bin/setDomainEnv.sh")
    # init-info
    replace_java_home("#{name}/init-info/domain-info.xml")
    replace_java_home("#{name}/init-info/startscript.xml")
    replace_java_home("#{name}/init-info/tokenValue.properties")
    replace_java_home("#{name}/init-info/nodemanager-properties.xml")
    # config
    replace_java_home("#{name}/config/config.xml")
    replace_java_home("#{name}/config/config_bootstrap.xml")
    # rest
    replace_java_home("#{name}/backup_config/backup_config.xml")
    replace_java_home("#{name}/servers/domain_bak/config_prev/config.xml")
    # Now check all the extra files
    extra_files.each { |f| replace_java_home(f, true) }
    nil
  end

  parameter :name
  parameter :extra_files
  property  :java_home
end
