# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'rexml/document'
module Puppet_X
  module EnterpriseModules
    module WebLogic
      # The puppet facts.
      module Facts
        def get_version(home)
          version = ''
          if File.exist?("#{home}/oui/mw/wls/metadata/install.properties")
            # rubocop:disable Security/Open
            open "#{home}/oui/mw/wls/metadata/install.properties" do |install_file|
              # rubocop:enable Security/Open
              install_file.each_line do |line|
                next if line.chomp.empty? || line =~ /^#/

                property, value = line.split('=')
                version = property == 'oracle.install.engine.release.version' ? value.chomp : ''
                break if version != ''
              end
            end
          end
          version
        end

        def get_installed_patches(oracle_home)
          ora_user = user_for_file(oracle_home)
          patches_in_home(oracle_home, ora_user).flatten.sort
        end

        def get_installed_features(oracle_product_home_dir)
          feature_files = Dir.glob("#{oracle_product_home_dir}/inventory/featuresets/*.xml")
          feature_files.collect { |file| File.basename(file, '.xml').gsub(/_\d.*/, '') }
        end

        def patches_in_home(oracle_product_home_dir, os_user)
          if opatch_installed?(oracle_product_home_dir)
            full_command = "#{oracle_product_home_dir}/OPatch/opatch lsinventory -oh #{oracle_product_home_dir}"
            raw_list = execute(full_command, os_user)
            raw_list.scan(/Patch\s.(\d+)\s.*:\sapplied on/).flatten
            # patch_ids.collect { |p| "#{oracle_product_home_dir}:#{p}" }
          else
            #
            # When Opatch is not installed, we don't report any patches
            #
            []
          end
        end

        def get_opatch_version(name)
          opatch_out = if File.exist?("#{name}/OPatch/opatch")
                         Facter::Util::Resolution.exec("#{name}/OPatch/opatch version")
                       else
                         ''
                       end
          opatchver = if opatch_out.nil?
                        'Error;'
                      else
                        opatch_out.split[2]
                      end
          Puppet.debug "wls_install opatch version #{name}/OPatch/opatch: #{opatchver}"
          opatchver
        end

        def get_running_domains(home)
          java_processes = Facter::Core::Execution.execute("stty cols 9999; ps -eaf | grep java | grep -v grep | grep #{home}")
          java_processes.match(%r{domains/(\w*?)/servers})&.captures&.uniq || []
        end

        def get_ora_inventory_path
          os = Facter.value(:kernel)
          case os
          when 'Linux'
            return '/etc'
          when 'SunOS'
            return '/var/opt/oracle'
          end

          '/etc'
        end

        def get_orainst_loc
          if File.exist?("#{get_ora_inventory_path}/oraInst.loc")
            str = ''
            output = File.read("#{get_ora_inventory_path}/oraInst.loc")
            output.split(/\r?\n/).each do |item|
              str = item[14, 50] if /^inventory_loc/.match?(item)
            end
            str
          else
            '/u01/app/oraInventory' # This is the default as used on market place images.
          end
        end

        def get_homes(path)
          if path && File.exist?("#{path}/ContentsXML/inventory.xml")
            file = File.read("#{path}/ContentsXML/inventory.xml")
            doc = REXML::Document.new(file)
            homes = []
            doc.elements.each('/INVENTORY/HOME_LIST/HOME') do |element|
              #
              # Skip all entryes that have subelements.
              #
              next if element.elements[1] && element.elements[1].name == 'DEPHOMELIST'

              str = element.attributes['LOC']
              if element.attributes['REMOVED'] == 'T'
                Puppet.debug "Home #{str} marked as removed in Oracle inventory; skipping for ora_install_homes fact."
                next
              end
              unless str.nil?
                if str.include? 'plugins'
                  # skip EM agent
                elsif str.include? 'jdk'
                  # skip EM agent
                elsif str.include? 'OraPlaceHolderDummyHome'
                  # skip EM agent
                else
                  homes << str.to_s
                end
              end
            end
            homes
          else
            {}
          end
        end

        def user_for_file(home)
          #
          # To run opatch, we need the owner of the grid and oracle directory. The mechanism
          # we use to get this user, is to fetch the owner of the bin directory in the grid or
          # oracle home. Sometimes however, these directories also contain files with root as owner.
          # since root can never be the user of the grid or oracle processen, we skip those.
          #
          a_file = first_non_root_file(home)
          uid = File.stat("#{home}/#{a_file}").uid
          Etc.getpwuid(uid).name
        end

        def first_non_root_file(dir)
          Dir.new(dir).each { |file| break file if File.stat("#{dir}/#{file}").uid.nonzero? }
        end

        def opatch_installed?(weblogic_home)
          File.exist?("#{weblogic_home}/OPatch/opatch")
        end

        def execute(command, user)
          full_command = if user
                           "su - #{user} -c \"#{command}\""
                         else
                           command
                         end
          `#{full_command}`
        end

        def oimconfigured?(root)
          provider = root.elements.to_a('security-configuration/realm//sec:name').first
          if provider.text == 'OIMAuthenticationProvider'
            Puppet.debug 'wls_install_homes.rb   found oim authentication provider'
            true
          else
            false
          end
        end

        def servers(root)
          servers = {}
          root.elements.each('server') do |server|
            name = server.elements['name']&.text
            port = server.elements['listen-port'].nil? ? '' : server.elements['listen-port'].text
            machine = server.elements['lmachine'].nil? ? '' : server.elements['machine'].text
            Puppet.debug "wls_install_homes.rb   found server '#{name}' machine '#{machine}' on port '#{port}'"
            servers[name] = { 'port' => port, 'machine' => machine }
          end
          servers.compact
        end

        def server_templates(root)
          server_templates = []
          root.elements.each('server-template') do |svr_template|
            name = svr_template.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found server template '#{name}'"
            server_templates << name
          end
          server_templates.compact
        end

        def clusters(root)
          clusters = []
          root.elements.each('cluster') do |cluster|
            name = cluster.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found cluster '#{name}'"
            clusters << name
          end
          clusters.compact
        end

        def coherence_clusters(root)
          coherence_clusters = []
          root.elements.each('coherence-cluster-system-resource') do |coherence|
            name = coherence.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found coherence cluster '#{name}'"
            coherence_clusters << name
          end
          coherence_clusters.compact
        end

        def deployments(root)
          deployments = []
          root.elements.each("app-deployment[module-type = 'ear']") do |apps|
            ear_name = apps.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found deployment '#{ear_name}'"
            deployments << ear_name
          end
          deployments.compact
        end

        def targets(root, type)
          targets = []
          root.elements.each("app-deployment[module-type = 'ear']") do |apps|
            ear_name = apps.elements['name']&.text
            ear_target_value = apps.elements['target']
            ear_target = ear_target_value.text.split(',') if ear_target_value&.text
            case type
            when 'bpm'
              if ear_name == 'BPMComposer'
                Puppet.debug "wls_install_homes.rb   found target(s) '#{ear_target}' for '#{type}'"
                targets << ear_target
              end
            when 'soa'
              if ear_name == 'soa-infra'
                Puppet.debug "wls_install_homes.rb   found target(s) '#{ear_target}' for '#{type}'"
                targets << ear_target
              end
            when 'osb'
              # rubocop: disable Performance/CollectionLiteralInLoop
              if ['ALSB Routing', 'Service Bus Routing'].include?(ear_name)
                Puppet.debug "wls_install_homes.rb   found target(s) '#{ear_target}' for '#{type}'"
                targets << ear_target
              end
              # rubocop: enable Performance/CollectionLiteralInLoop
            end
          end
          targets.flatten.compact.uniq
        end

        def adapters(root)
          file_adapter_plan = []
          file_adapter_plan_entries = []
          root.elements.each("app-deployment[name = 'FileAdapter']") do |apps|
            next if apps.elements['plan-path'].nil?

            plan = if apps.elements['plan-dir'].attributes['xsi:nil'] == 'true'
                     apps.elements['plan-path'].text
                   else
                     "#{apps.elements['plan-dir'].text}/#{apps.elements['plan-path'].text}"
                   end
            Puppet.debug "wls_install_homes.rb   found file_adapter_plan '#{plan}'"
            file_adapter_plan << plan
            next unless File.exist?(plan)

            subfile = File.read(file_adapter_plan)
            subdoc = REXML::Document.new subfile
            planroot = subdoc.root
            planroot.elements['variable-definition'].elements.each('variable') do |eis|
              entry = eis.elements['value'].text
              if entry.include? 'eis'
                Puppet.debug "wls_install_homes.rb   found file_adapter_entry '#{entry}'"
                file_adapter_plan_entries << entry
              end
            end
          end

          db_adapter_plan = []
          db_adapter_plan_entries = []
          root.elements.each("app-deployment[name = 'DbAdapter']") do |apps|
            next if apps.elements['plan-path'].nil?

            plan = if apps.elements['plan-dir'].attributes['xsi:nil'] == 'true'
                     apps.elements['plan-path'].text
                   else
                     "#{apps.elements['plan-dir'].text}/#{apps.elements['plan-path'].text}"
                   end
            Puppet.debug "wls_install_homes.rb   found db_adapter_plan '#{plan}'"
            db_adapter_plan << plan

            next unless File.exist?(plan)

            subfile = File.read(plan)
            subdoc = REXML::Document.new subfile
            planroot = subdoc.root
            planroot.elements['variable-definition'].elements.each('variable') do |eis|
              entry = eis.elements['value'].text
              if !entry.nil? && entry.include?('eis')
                Puppet.debug "wls_install_homes.rb   found db_adapter_entry '#{entry}'"
                db_adapter_plan_entries << entry
              end
            end
          end

          aq_adapter_plan = []
          aq_adapter_plan_entries = []
          root.elements.each("app-deployment[name = 'AqAdapter']") do |apps|
            next if apps.elements['plan-path'].nil?

            plan = if apps.elements['plan-dir'].attributes['xsi:nil'] == 'true'
                     apps.elements['plan-path'].text
                   else
                     "#{apps.elements['plan-dir'].text}/#{apps.elements['plan-path'].text}"
                   end
            Puppet.debug "wls_install_homes.rb   found aq_adapter_plan '#{plan}'"
            aq_adapter_plan << plan

            next unless File.exist?(plan)

            subfile = File.read(plan)
            subdoc = REXML::Document.new subfile
            planroot = subdoc.root
            planroot.elements['variable-definition'].elements.each('variable') do |eis|
              entry = eis.elements['value'].text
              if !entry.nil? && entry.include?('eis')
                Puppet.debug "wls_install_homes.rb   found aq_adapter_entry '#{entry}'"
                aq_adapter_plan_entries << entry
              end
            end
          end

          jms_adapter_plan = []
          jms_adapter_plan_entries = []
          root.elements.each("app-deployment[name = 'JmsAdapter']") do |apps|
            next if apps.elements['plan-path'].nil?

            plan = if apps.elements['plan-dir'].attributes['xsi:nil'] == 'true'
                     apps.elements['plan-path'].text
                   else
                     "#{apps.elements['plan-dir'].text}/#{apps.elements['plan-path'].text}"
                   end
            Puppet.debug "wls_install_homes.rb   found jms_adapter_plan '#{plan}'"
            jms_adapter_plan << plan

            next unless File.exist?(plan)

            subfile = File.read(plan)
            subdoc = REXML::Document.new subfile
            planroot = subdoc.root
            planroot.elements['variable-definition'].elements.each('variable') do |eis|
              entry = eis.elements['value'].text
              if !entry.nil? && entry.include?('eis')
                Puppet.debug "wls_install_homes.rb   found jms_adapter_entry '#{entry}'"
                jms_adapter_plan_entries << entry
              end
            end
          end

          ftp_adapter_plan = []
          ftp_adapter_plan_entries = []
          root.elements.each("app-deployment[name = 'FtpAdapter']") do |apps|
            next if apps.elements['plan-path'].nil?

            plan = if apps.elements['plan-dir'].attributes['xsi:nil'] == 'true'
                     apps.elements['plan-path'].text
                   else
                     "#{apps.elements['plan-dir'].text}/#{apps.elements['plan-path'].text}"
                   end
            Puppet.debug "wls_install_homes.rb   found ftp_adapter_plan '#{plan}'"
            ftp_adapter_plan << plan

            next unless File.exist?(plan)

            subfile = File.read(plan)
            subdoc = REXML::Document.new subfile
            planroot = subdoc.root
            planroot.elements['variable-definition'].elements.each('variable') do |eis|
              entry = eis.elements['value'].text
              if !entry.nil? && entry.include?('eis')
                Puppet.debug "wls_install_homes.rb   found ftp_adapter_entry '#{entry}'"
                ftp_adapter_plan_entries << entry
              end
            end
          end
          [file_adapter_plan.compact,
           file_adapter_plan_entries.compact,
           db_adapter_plan.compact,
           db_adapter_plan_entries.compact,
           aq_adapter_plan.compact,
           aq_adapter_plan_entries.compact,
           jms_adapter_plan.compact,
           jms_adapter_plan_entries.compact,
           ftp_adapter_plan.compact,
           ftp_adapter_plan_entries.compact]
        end

        def libraries(root)
          jrf_target  = []
          bam_target  = []
          libraries   = []
          root.elements.each('library') do |libs|
            lib_name = libs.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found library '#{lib_name}'"
            libraries << lib_name
            if lib_name.include? 'adf.oracle.domain#1.0'
              target = libs.elements['target'].text.split(',')
              jrf_target << target
              Puppet.debug "wls_install_homes.rb   found target(s) '#{target}' for jrf"
            end
            next unless lib_name.include? 'oracle.bam.library'

            target = libs.elements['target'].text.split(',')
            bam_target << target
            Puppet.debug "wls_install_homes.rb   found target(s) '#{target}' for bam"
          end
          [jrf_target.flatten.compact.uniq,
           bam_target.flatten.compact.uniq,
           libraries.compact]
        end

        def filestores(root)
          filestores = []
          root.elements.each('file-store') do |filestore|
            store = filestore.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found filestore '#{store}'"
            filestores << store
          end
          filestores.compact
        end

        def jdbcstores(root)
          jdbcstores = []
          root.elements.each('jdbc-store') do |jdbc|
            store = jdbc.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found jdbcstore '#{store}'"
            jdbcstores << store
          end
          jdbcstores.compact
        end

        def safagents(root)
          safagents = []
          root.elements.each('saf-agent') do |agent|
            agent = agent.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found safagent '#{agent}'"
            safagents << agent
          end
          safagents.compact
        end

        def jmsservers(root)
          jmsserversstr = []
          root.elements.each('jms-server') do |jmsservers|
            server = jmsservers.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found jmsserver '#{server}'"
            jmsserversstr << server
          end
          jmsserversstr.compact
        end

        def jmsmodules(root, domain_path)
          jmsmodules_listing = root.elements.collect('jms-system-resource') do |jmsresource|
            jmsstr = []
            jmssubdeployments = []

            jmsresource.elements.each('sub-deployment') do |sub|
              deployment = sub.elements['name']&.text
              Puppet.debug "wls_install_homes.rb   found sub-deployment '#{deployment}'"
              jmssubdeployments << deployment
            end

            file_jms = if jmsresource.elements['descriptor-file-name'].nil?
                         "jms/#{jmsresource.elements['name'].text.downcase}-jms.xml"
                       else
                         jmsresource.elements['descriptor-file-name'].text
                       end

            subfile = File.read("#{domain_path}/config/#{file_jms}")
            subdoc = REXML::Document.new subfile

            jmsroot = subdoc.root

            jmsmodule_quota_str = []
            jmsroot.elements.each('quota') do |qu|
              quota = qu.attributes['name']
              Puppet.debug "wls_install_homes.rb   found jmsmodule quota '#{quota}'"
              jmsmodule_quota_str << quota
            end

            jmsmodule_foreign_server_str = {}
            jmsroot.elements.each('foreign-server') do |fs|
              fs_name = fs.attributes['name']
              Puppet.debug "wls_install_homes.rb   found foreign server '#{fs_name}'"

              jmsmodule_foreign_server_objects_str = []
              fs.elements.each('foreign-destination') do |cf|
                object = cf.attributes['name']
                Puppet.debug "wls_install_homes.rb     found object '#{object}'"
                jmsmodule_foreign_server_objects_str << object
              end
              fs.elements.each('foreign-connection-factory') do |dest|
                object = dest.attributes['name']
                Puppet.debug "wls_install_homes.rb     found object '#{object}'"
                jmsmodule_foreign_server_objects_str << object
              end
              jmsmodule_foreign_server_str = [fs_name, { 'objects' => jmsmodule_foreign_server_objects_str }]
            end

            jmsmodule_name = jmsresource.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found jms module '#{jmsmodule_name}'"

            jmsroot.elements.each('connection-factory') do |cfs|
              connection_factory = cfs.attributes['name']
              Puppet.debug "wls_install_homes.rb     found connection-factory '#{connection_factory}'"
              jmsstr << connection_factory
            end

            jmsroot.elements.each('queue') do |queues|
              queue = queues.attributes['name']
              Puppet.debug "wls_install_homes.rb     found queue '#{queue}'"
              jmsstr << queue
            end

            jmsroot.elements.each('uniform-distributed-queue') do |dist_queues|
              queue = dist_queues.attributes['name']
              Puppet.debug "wls_install_homes.rb     found uniform-distributed-queue '#{queue}'"
              jmsstr << queue
            end

            jmsroot.elements.each('topic') do |topics|
              topic = topics.attributes['name']
              Puppet.debug "wls_install_homes.rb     found topic '#{topic}'"
              jmsstr << topic
            end

            jmsroot.elements.each('uniform-distributed-topic') do |dist_topics|
              topic = dist_topics.attributes['name']
              Puppet.debug "wls_install_homes.rb     found uniform-distributed-topic '#{topic}'"
              jmsstr << topic
            end

            [jmsmodule_name, { 'foreign_servers' => jmsmodule_foreign_server_str,
                               'objects' => jmsstr,
                               'quotas' => jmsmodule_quota_str,
                               'subdeployments' => jmssubdeployments }]
          end
          jmsmodules_listing.to_h
        end

        def jdbc(root)
          jdbcstr = []
          root.elements.each('jdbc-system-resource') do |jdbcresource|
            jdbc = jdbcresource.elements['name']&.text
            Puppet.debug "wls_install_homes.rb   found jdbc resource '#{jdbc}'"
            jdbcstr << jdbc
          end
          jdbcstr.compact
        end

        # read weblogic domain
        def domains_hash
          domainfile = '/etc/wls_domains.dat'

          unless File.exist?(domainfile)
            Puppet.debug "wls_install_homes.rb #{domainfile} not found"
            return {}
          end

          file = File.read(domainfile)
          domain_array = file.split.map { |domain| domain.split(':') }.flatten
          domains = Hash[*domain_array.flatten]

          domain_listing = domains.keys.collect do |domain_name|
            path = domains[domain_name]
            Puppet.debug "wls_install_homes.rb found domain '#{domain_name}' path '#{path}'"
            if File.exist?("#{path}/config/config.xml")
              domain_config_file = "#{path}/config/config.xml"
            elsif File.directory?("#{path}/config")
              return { domain_name => { 'path' => path } }
            else
              return {} # wls_domains exist, but directory is removed.
            end
            file = File.read(domain_config_file)
            doc = REXML::Document.new file
            root = doc.root
            if root
              fileadapter_plan, fileadapter_entries, db_adapter_plan,
                db_adapter_entries, aq_adapter_plan, aq_adapter_entries, \
                jms_adapter_plan, jms_adapter_entries, ftp_adapter_plan, \
                ftp_adapter_entries = adapters(root)

              jrf_targets, bam_targets, libraries = libraries(root)

              [domain_name, { 'bam' => bam_targets,
                              'bpm' => targets(root, 'bpm'),
                              'clusters' => clusters(root),
                              'coherence_clusters' => coherence_clusters(root),
                              'deployments' => deployments(root),
                              'eis_aqadapter_entries' => aq_adapter_entries,
                              'eis_aqadapter_plan' => aq_adapter_plan,
                              'eis_dbadapter_entries' => db_adapter_entries,
                              'eis_dbadapter_plan' => db_adapter_plan,
                              'eis_fileadapter_entries' => fileadapter_entries,
                              'eis_fileadapter_plan' => fileadapter_plan,
                              'eis_ftpadapter_entries' => ftp_adapter_entries,
                              'eis_ftpadapter_plan' => ftp_adapter_plan,
                              'eis_jmsadapter_entries' => jms_adapter_entries,
                              'eis_jmsadapter_plan' => jms_adapter_plan,
                              'filestores' => filestores(root),
                              'jdbc' => jdbc(root),
                              'jdbcstores' => jdbcstores(root),
                              'jmsmodules' => jmsmodules(root, path),
                              'jmsservers' => jmsservers(root),
                              'jrf' => jrf_targets,
                              'libraries' => libraries,
                              'oimconfigured' => oimconfigured?(root),
                              'osb' => targets(root, 'osb'),
                              'path' => path,
                              'safagents' => safagents(root),
                              'servers' => servers(root),
                              'soa' => targets(root, 'soa'),
                              'templates' => server_templates(root) }]
            else
              [domain_name, { 'bam' => [],
                              'bpm' => [],
                              'clusters' => [],
                              'coherence_clusters' => [],
                              'deployments' => [],
                              'eis_aqadapter_entries' => [],
                              'eis_aqadapter_plan' => [],
                              'eis_dbadapter_entries' => [],
                              'eis_dbadapter_plan' => [],
                              'eis_fileadapter_entries' => [],
                              'eis_fileadapter_plan' => [],
                              'eis_ftpadapter_entries' => [],
                              'eis_ftpadapter_plan' => [],
                              'eis_jmsadapter_entries' => [],
                              'eis_jmsadapter_plan' => [],
                              'filestores' => [],
                              'jdbc' => [],
                              'jdbcstores' => [],
                              'jmsmodules' => {},
                              'jmsservers' => [],
                              'jrf' => [],
                              'libraries' => [],
                              'oimconfigured' => false,
                              'osb' => [],
                              'path' => path,
                              'safagents' => [],
                              'servers' => {},
                              'soa' => [],
                              'templates' => [] }]
            end
          end
          domain_listing.sort.to_h
        end
      end
    end
  end
end
