#
# wls_install::utils::fmwclusterjrf
#
# @summary This defined type allows you to convert an existing cluster into an ADF/JRF cluster.
# Here is an example on how you could use this:
# 
# ```
# ora_install::utils::fmwclusterjrf { 'WebCluster':
#   domain_name          => "adf_domain",
#   jrf_target_name      => "WebCluster",
#   opss_datasource_name => "opss-data-source",
# }
# 
# ```
#
# @param [Wls_install::Versions] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Stdlib::Absolutepath] weblogic_home_dir
#    Directory to use as WebLogic home.
#
# @param [Stdlib::Absolutepath] middleware_home_dir
#    Middleware home directory.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [String[1]] domain_name
#    The domain name.
#
# @param [Stdlib::Absolutepath] wls_domains_dir
#    The root directory where all domain directories reside in.
#
# @param [String[1]] adminserver_name
#    The name of the Admin Server.
#
# @param [String[1]] adminserver_address
#    The address of the admin server.
#    May be a DNS name or a physical IP address.
#
# @param [Integer] adminserver_port
#    The IP port the admin server is listening on.
#
# @param [Integer] nodemanager_port
#    The IP port the nodemanager is listening on.
#
# @param [String[1]] jrf_target_name
#    Target to use for JRF objects.
#
# @param [String[1]] opss_datasource_name
#    Datasource to use OPSS.
#
# @param [String[1]] weblogic_user
#    The base WebLogic username to configure.
#
# @param [Easy_type::Password] weblogic_password
#    The password of the base WebLogic user name.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Variant[Boolean, Enum['on_failure', ]]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::utils::fmwclusterjrf (
  Wls_install::Versions $version                    = lookup({ name => 'wls_version', default_value => 1111 }),
  Stdlib::Absolutepath  $weblogic_home_dir          = lookup({ name => 'wls_weblogic_home_dir' }),
  Stdlib::Absolutepath  $middleware_home_dir        = lookup({ name => 'wls_middleware_home_dir' }),
  Stdlib::Absolutepath  $jdk_home_dir               = lookup({ name => 'wls_jdk_home_dir' }),
  String[1]             $domain_name                = lookup({ name => 'domain_name' }),
  Stdlib::Absolutepath  $wls_domains_dir            = lookup({ name => 'wls_domains_dir', default_value => undef }),
  String[1]             $adminserver_name           = lookup({ name => 'domain_adminserver', default_value => 'AdminServer' }),
  String[1]             $adminserver_address        = lookup({ name => 'domain_adminserver_address', default_value => 'localhost' }),
  Integer               $adminserver_port           = lookup({ name => 'domain_adminserver_port', default_value => 7001 }),
  Integer               $nodemanager_port           = lookup({ name => 'domain_nodemanager_port', default_value => 5556 }),
  String[1]             $jrf_target_name            = undef,
  String[1]             $opss_datasource_name       = undef,
  String[1]             $weblogic_user              = lookup({ name => 'wls_weblogic_user', default_value => 'weblogic' }),
  Easy_type::Password   $weblogic_password          = lookup({ name => 'domain_wls_password' }),
  String[1]             $os_user                    = lookup({ name => 'wls_os_user' }),
  String[1]             $os_group                   = lookup({ name => 'wls_os_group' }),
  Stdlib::Absolutepath  $download_dir               = lookup({ name => 'wls_download_dir' }),
  Variant[
    Boolean,
    Enum[
      'on_failure',
    ]
  ]                     $logoutput                  = lookup({ name => 'logoutput', default_value => 'on_failure' }),
) {
  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }
  $domain_dir = "${domains_dir}/${domain_name}"

  easy_type::debug_evaluation() # Show local variable on extended debug

  unless ( wls_install::jrf_cluster_configured($domain_dir, $jrf_target_name) ) {
    #shutdown adminserver for offline WLST scripts
    wls_install::control { "ShutdownAdminServerForJRF${title}":
      weblogic_home_dir   => $weblogic_home_dir,
      jdk_home_dir        => $jdk_home_dir,
      domain_name         => $domain_name,
      wls_domains_dir     => $domains_dir,
      middleware_home_dir => $middleware_home_dir,
      server_type         => 'admin',
      target              => 'Server',
      server              => $adminserver_name,
      adminserver_address => $adminserver_address,
      adminserver_port    => $adminserver_port,
      nodemanager_port    => $nodemanager_port,
      action              => 'stop',
      weblogic_user       => $weblogic_user,
      weblogic_password   => $weblogic_password,
      os_user             => $os_user,
      os_group            => $os_group,
      download_dir        => $download_dir,
      logoutput           => $logoutput,
    }

    file { "${download_dir}/${title}_assignJrfToCluster.py":
      ensure  => file,
      content => epp('wls_install/wlst/wlstexec/fmw/assignJrfToCluster.py.epp', {
          'version'              => $version,
          'weblogic_home_dir'    => $weblogic_home_dir,
          'domain_dir'           => $domain_dir,
          'jrf_target_name'      => $jrf_target_name,
          'opss_datasource_name' => $opss_datasource_name,
      }),
      backup  => false,
      replace => true,
      mode    => '0775',
      owner   => $os_user,
      group   => $os_group,
    }

    $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

    # reorder all apps,libraries, startup , shutdown
    exec { "execwlst assignJrfToCluster.py ${title}":
      command     => "${middleware_home_dir}/oracle_common/common/bin/wlst.sh ${download_dir}/${title}_assignJrfToCluster.py",
      environment => [
        "JAVA_HOME=${jdk_home_dir}",
      ],
      path        => $exec_path,
      user        => $os_user,
      group       => $os_group,
      logoutput   => $logoutput,
      require     => [
        File["${download_dir}/${title}_assignJrfToCluster.py"],
        Wls_install::Control["ShutdownAdminServerForJRF${title}"]
      ],
    }

    #startup adminserver for offline WLST scripts
    wls_install::control { "StartupAdminServerForJSF${title}":
      weblogic_home_dir   => $weblogic_home_dir,
      jdk_home_dir        => $jdk_home_dir,
      domain_name         => $domain_name,
      wls_domains_dir     => $domains_dir,
      middleware_home_dir => $middleware_home_dir,
      server_type         => 'admin',
      target              => 'Server',
      server              => $adminserver_name,
      adminserver_address => $adminserver_address,
      adminserver_port    => $adminserver_port,
      nodemanager_port    => $nodemanager_port,
      action              => 'start',
      weblogic_user       => $weblogic_user,
      weblogic_password   => $weblogic_password,
      os_user             => $os_user,
      os_group            => $os_group,
      download_dir        => $download_dir,
      logoutput           => $logoutput,
      require             => Exec["execwlst assignJrfToCluster.py ${title}"],
    }
  }
}
