#
# wls_install::utils::webtier
#
# @summary This defined type allows you to add an OHS instance to a WebLogic Domain and in the Enterprise Manager, optional with OHS OAM Webgate.
# Here is an example on how you could use this:
# 
# ```
# ora_install::utils::webtier{ 'ohs1':
#   action_name       => 'create',
#   instance_name     => 'ohs1',
#   webgate_configure => true,
# }
# ```
#
# @param [Wls_install::Versions] version
#    The version that is installed in the used Oracle home.
#    Puppet uses this value to decide on version specific actions.
#
# @param [Stdlib::Absolutepath] weblogic_home_dir
#    Directory to use as WebLogic home.
#
# @param [Stdlib::Absolutepath] middleware_home_dir
#    Middleware home directory.
#
# @param [Stdlib::Absolutepath] jdk_home_dir
#    The homdirectory for the Java evelopment Kit.
#
# @param [Stdlib::Absolutepath] wls_domains_dir
#    The root directory where all domain directories reside in.
#
# @param [String[1]] domain_name
#    The domain name.
#
# @param [String[1]] adminserver_address
#    The address of the admin server.
#    May be a DNS name or a physical IP address.
#
# @param [Integer] adminserver_port
#    The IP port the admin server is listening on.
#
# @param [Enum['create', 'delete']] action_name
#    Action to perform.
#    Valid values are:
#    - create
#    - delete
#
# @param [Boolean] webgate_configure
#    
#
# @param [Optional[String[1]]] webgate_agentname
#    
#
# @param [Optional[String[1]]] webgate_hostidentifier
#    
#
# @param [String[1]] oamadminserverhostname
#    
#
# @param [Integer] oamadminserverport
#    
#
# @param [Boolean] domain_configure
#    
#
# @param [Optional[String[1]]] instance_name
#    
#
# @param [Optional[String[1]]] machine_name
#    Name of the weblogic machine to use.
#
# @param [String[1]] weblogic_user
#    The base WebLogic username to configure.
#
# @param [Easy_type::Password] weblogic_password
#    The password of the base WebLogic user name.
#
# @param [String[1]] os_user
#    The os user used to install and run WebLogic.
#
# @param [String[1]] os_group
#    The os group used to install and run WebLogic.
#
# @param [Stdlib::Absolutepath] download_dir
#    The directory where the Puppet software puts all downloaded files.
#    Before Puppet can actually use remote files, they must be downloaded first. Puppet uses this directory to put all files in.
#    The default value is: `/install`
#
# @param [Variant[Boolean, Enum['on_failure', ]]] logoutput
#    log the outputs of Puppet exec or not.
#    When you specify `true` Puppet will log all output of `exec` types.
#    Valid values are:
#    - `true`
#    - `false`
#    - `on_failure`
#
#
# See the file "LICENSE" for the full license governing this code.
#
define wls_install::utils::webtier (
  Wls_install::Versions   $version                    = lookup({ name => 'wls_version', default_value => 1111 }),
  Stdlib::Absolutepath    $weblogic_home_dir          = lookup({ name => 'wls_weblogic_home_dir' }),
  Stdlib::Absolutepath    $middleware_home_dir        = lookup({ name => 'wls_middleware_home_dir' }),
  Stdlib::Absolutepath    $jdk_home_dir               = lookup({ name => 'wls_jdk_home_dir' }),
  Stdlib::Absolutepath    $wls_domains_dir            = lookup({ name => 'wls_domains_dir', default_value => undef }),
  String[1]               $domain_name                = lookup({ name => 'domain_name' }),
  String[1]               $adminserver_address        = lookup({ name => 'domain_adminserver_address', default_value => 'localhost' }),
  Integer                 $adminserver_port           = lookup({ name => 'domain_adminserver_port', default_value => 7001 }),
  Enum['create','delete'] $action_name                = 'create',
  Boolean                 $webgate_configure          = false,
  Optional[String[1]]     $webgate_agentname          = lookup({ name => 'webgate_agentname', default_value => undef }),
  Optional[String[1]]     $webgate_hostidentifier     = lookup({ name => 'webgate_hostidentifier', default_value => undef }),
  String[1]               $oamadminserverhostname     = lookup({ name => 'oamadminserverhostname', default_value => 'localhost' }),
  Integer                 $oamadminserverport         = lookup({ name => 'oamadminserverport', default_value => 7001 }),
  Boolean                 $domain_configure           = true,
  Optional[String[1]]     $instance_name              = undef,
  Optional[String[1]]     $machine_name               = undef,
  String[1]               $weblogic_user              = lookup({ name => 'wls_weblogic_user', default_value => 'weblogic' }),
  Easy_type::Password     $weblogic_password          = lookup({ name => 'domain_wls_password' }),
  String[1]               $os_user                    = lookup({ name => 'wls_os_user' }),
  String[1]               $os_group                   = lookup({ name => 'wls_os_group' }),
  Stdlib::Absolutepath    $download_dir               = lookup({ name => 'wls_download_dir' }),
  Variant[
    Boolean,
    Enum[
      'on_failure',
    ]
  ]                       $logoutput                   = lookup({ name => 'logoutput', default_value => 'on_failure' }),
) {
  if ( $wls_domains_dir ) {
    $domains_dir =  $wls_domains_dir
  } else {
    $domains_dir = "${middleware_home_dir}/user_projects/domains"
  }
  $domain_dir = "${domains_dir}/${domain_name}"

  $exec_path = "${jdk_home_dir}/bin:/usr/local/bin:/bin:/usr/bin:/usr/local/sbin:/usr/sbin:/sbin:"

  easy_type::debug_evaluation() # Show local variable on extended debug

  if ( $version == 1212 or $version == 1213 or $version >= 1221 ) {
    file { "${download_dir}/${title}_createWebtier.py":
      ensure    => file,
      content   => epp('wls_install/wlst/wlstexec/fmw/createWebtier.py.epp', {
          'weblogic_user'       => $weblogic_user,
          'adminserver_address' => $adminserver_address,
          'adminserver_port'    => $adminserver_port,
          'action_name'         => $action_name,
          'instance_name'       => $instance_name,
          'machine_name'        => $machine_name,
      }),
      backup    => false,
      show_diff => false,
      replace   => true,
      mode      => '0775',
      owner     => $os_user,
      group     => $os_group,
    }

    if ( $action_name == 'create' ) {
      exec { "execwlst webtier ${title}":
        command     => "${middleware_home_dir}/ohs/common/bin/wlst.sh ${download_dir}/${title}_createWebtier.py ${weblogic_password.unwrap}",
        environment => "JAVA_HOME=${jdk_home_dir}",
        path        => $exec_path,
        creates     => "${domain_dir}/system_components/OHS/${instance_name}",
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => File["${download_dir}/${title}_createWebtier.py"],
      }
    } else {
      exec { "execwlst webtier ${title}":
        command     => "${middleware_home_dir}/ohs/common/bin/wlst.sh ${download_dir}/${title}_createWebtier.py ${weblogic_password.unwrap}",
        environment => "JAVA_HOME=${jdk_home_dir}",
        path        => $exec_path,
        onlyif      => "test -d ${domain_dir}/system_components/OHS/${instance_name}",
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => File["${download_dir}/${title}_createWebtier.py"],
      }
    }
  } else {
    $instance_home = "${middleware_home_dir}/Oracle_WT1/instances/${instance_name}"
    $instance_id = "${middleware_home_dir}/Oracle_WT1/instances/${instance_name}/config/OHS/${instance_name}"

    file { "${download_dir}/${title}_configureWebtier.rsp":
      ensure    => file,
      content   => epp('wls_install/wlst/wlstexec/fmw/configureWebtier.rsp.epp', {
          'instance_home'       => $instance_home,
          'instance_name'       => $instance_name,
          'domain_configure'    => $domain_configure,
          'adminserver_address' => $adminserver_address,
          'adminserver_port'    => $adminserver_port,
          'weblogic_user'       => $weblogic_user,
          'weblogic_password'   => $weblogic_password,
      }),
      backup    => false,
      show_diff => false,
      replace   => true,
      mode      => '0775',
      owner     => $os_user,
      group     => $os_group,
    }

    exec { "config webtier ${title}":
      command     => "/bin/sh -c 'unset DISPLAY;${middleware_home_dir}/Oracle_WT1/bin/config.sh -silent -response ${download_dir}/${title}_configureWebtier.rsp -waitforcompletion'",
      environment => "JAVA_HOME=${jdk_home_dir}",
      path        => $exec_path,
      creates     => "${middleware_home_dir}/Oracle_WT1/instances/${instance_name}",
      user        => $os_user,
      group       => $os_group,
      logoutput   => $logoutput,
      require     => File["${download_dir}/${title}_configureWebtier.rsp"],
    }

    if ( $webgate_configure ) {
      # Note: rreg really should NOT be run from the "WebGate" machine
      # instead it should be run on the OAM server, the output artifacts stored in a secure place,
      # and then copied from there upon installation of the WebGate
      # for now this is an OK start

      # first create the input file:
      file { "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/input/${webgate_agentname}.xml":
        ensure    => file,
        content   => epp('wls_install/oam/rreg_input.xml.epp', {
            'oamadminserverhostname' => $oamadminserverhostname,
            'oamadminserverport'     => $oamadminserverport,
            'webgate_hostidentifier' => $webgate_hostidentifier,
            'webgate_agentname'      => $webgate_agentname,
        }),
        backup    => false,
        show_diff => false,
        replace   => true,
        mode      => '0440',
        owner     => $os_user,
        group     => $os_group,
      }

      file { "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/bin/oamreg.sh":
        ensure => file,
        mode   => '0700',
      }

      # in the real world rreg wouldn't be run on the host running the WebTier bits
      exec { "Run rreg for WebGate instance ${webgate_agentname}":
        command     => "/bin/echo -e ${weblogic_user}\\\\n${weblogic_password.unwrap}\\\\n|${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/bin/oamreg.sh inband input/${webgate_agentname}.xml -noprompt",
        environment => "JAVA_HOME=${jdk_home_dir}",
        cwd         => "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg",
        creates     => "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/output/${webgate_agentname}",
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          File["${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/input/${webgate_agentname}.xml"],
          File["${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/bin/oamreg.sh"]
        ],
      }

      exec { "Deploy webgate to ${instance_id}":
        command     => "${middleware_home_dir}/Oracle_OAMWebGate1/webgate/ohs/tools/deployWebGate/deployWebGateInstance.sh -w ${instance_id} -oh ${middleware_home_dir}/Oracle_OAMWebGate1",
        environment => "LD_LIBRARY_PATH='${middleware_home_dir}/Oracle_WT1/lib'",
        cwd         => "${middleware_home_dir}/Oracle_OAMWebGate1/webgate/ohs/tools/deployWebGate",
        creates     => "${instance_id}/webgate",
        path        => $exec_path,
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          Exec["config webtier ${title}"],
          Exec["Run rreg for WebGate instance ${webgate_agentname}"]
        ],
      }

      # copy the output from rreg to the WebGate
      # in the real world the rreg artifacts would:
      # * be copied to this host during host provisioning
      #   OR
      # * would be stored on a fileserver accessible from this host
      #   e.g. from puppet:
      file { "Copy rreg artifacts for ${instance_id}":
        path    => "${instance_id}/webgate/config",
        source  => "${middleware_home_dir}/Oracle_IDM1/oam/server/rreg/output/${webgate_agentname}",
        recurse => true,
        require => [
          Exec["Deploy webgate to ${instance_id}"],
          Exec["Run rreg for WebGate instance ${webgate_agentname}"]
        ],
      }

      exec { "EditHttpConf to enable webgate ${title}":
        command     => "${middleware_home_dir}/Oracle_OAMWebGate1/webgate/ohs/tools/setup/InstallTools/EditHttpConf -w ${instance_id} -oh ${middleware_home_dir}/Oracle_OAMWebGate1",
        environment => [
          "LD_LIBRARY_PATH=${middleware_home_dir}/Oracle_WT1/lib",
          "ORACLE_HOME=${middleware_home_dir}/Oracle_OAMWebGate1",
        ],
        cwd         => "${middleware_home_dir}/Oracle_OAMWebGate1/webgate/ohs/tools/setup/InstallTools",
        path        => $exec_path,
        unless      => "grep -c '${instance_id}/webgate.conf' ${instance_id}/httpd.conf",
        user        => $os_user,
        group       => $os_group,
        logoutput   => $logoutput,
        require     => [
          Exec["Deploy webgate to ${instance_id}"],
          File["Copy rreg artifacts for ${instance_id}"]
        ],
      }

      # after enabling a WebGate you need to bounce OHS
      exec { "Stop OHS instance ${title}":
        command   => "/bin/sh -c 'unset DISPLAY;${middleware_home_dir}/Oracle_WT1/instances/${instance_name}/bin/opmnctl stopall'",
        path      => $exec_path,
        user      => $os_user,
        group     => $os_group,
        logoutput => $logoutput,
        require   => Exec["EditHttpConf to enable webgate ${title}"],
      }

      exec { "Start OHS instance ${title}":
        command   => "/bin/sh -c 'unset DISPLAY;${middleware_home_dir}/Oracle_WT1/instances/${instance_name}/bin/opmnctl startall'",
        path      => $exec_path,
        user      => $os_user,
        group     => $os_group,
        logoutput => $logoutput,
        require   => Exec["Stop OHS instance ${title}"],
      }
    }
  }
}
